'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _desc, _value, _class, _descriptor;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _mobx = require('mobx');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _initDefineProp(target, property, descriptor, context) {
  if (!descriptor) return;
  Object.defineProperty(target, property, {
    enumerable: descriptor.enumerable,
    configurable: descriptor.configurable,
    writable: descriptor.writable,
    value: descriptor.initializer ? descriptor.initializer.call(context) : void 0
  });
}

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _initializerWarningHelper(descriptor, context) {
  throw new Error('Decorating class property failed. Please ensure that transform-class-properties is enabled.');
}

function _applyDecoratedDescriptor(target, property, decorators, descriptor, context) {
  var desc = {};
  Object['ke' + 'ys'](descriptor).forEach(function (key) {
    desc[key] = descriptor[key];
  });
  desc.enumerable = !!desc.enumerable;
  desc.configurable = !!desc.configurable;

  if ('value' in desc || desc.initializer) {
    desc.writable = true;
  }

  desc = decorators.slice().reverse().reduce(function (desc, decorator) {
    return decorator(target, property, desc) || desc;
  }, desc);

  if (context && desc.initializer !== void 0) {
    desc.value = desc.initializer ? desc.initializer.call(context) : void 0;
    desc.initializer = undefined;
  }

  if (desc.initializer === void 0) {
    Object['define' + 'Property'](target, property, desc);
    desc = null;
  }

  return desc;
}

/**
 * Generate a "unique" ID with an optional prefix
 * @param prefix
 * @returns {string}
 */
function randomId() {
  var prefix = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'id';

  var integer = Math.round(Math.random() * Number.MAX_SAFE_INTEGER);
  return prefix + '-' + integer;
}
/**
 * Base class for managing the flow of multiple steps.
 * Must provide an initial step, and has methods for pushing or replacing steps on stack.
 */
var FlowManager = (_class = function () {
  function FlowManager() {
    _classCallCheck(this, FlowManager);

    _initDefineProp(this, 'stateId', _descriptor, this);

    this.states = [];
    this.steps = [];
    this.placeholders = [];
  }

  _createClass(FlowManager, [{
    key: 'initialize',


    /**
     * Sets up the initial state of the flow. Do not override.
     * @param listener
     */
    value: function initialize(listener) {
      this._reset();

      this.listener = listener;

      this._setupStates();
      this._setupInitialStep();
      this.onInitialized();
    }
  }, {
    key: 'getStates',
    value: function getStates() {
      this._throwError('needs valid state list');
    }

    /**
     * Return a React component (enclosed by FlowStep) that starts the flow
     */

  }, {
    key: 'getInitialStep',
    value: function getInitialStep() {
      this._throwError('need initial step');
    }

    /**
     * Callback invoked after initial step is set up
     */

  }, {
    key: 'onInitialized',
    value: function onInitialized() {}

    // new APIS

  }, {
    key: 'renderStep',


    /**
     * Render the specified state and step.
     *
     * @param stateId
     * @param stepElement
     * @param afterStateId
     */
    value: function renderStep(_ref) {
      var stateId = _ref.stateId,
          stepElement = _ref.stepElement,
          _ref$afterStateId = _ref.afterStateId,
          afterStateId = _ref$afterStateId === undefined ? null : _ref$afterStateId;

      if (!stateId || !stepElement) {
        this._throwError('stateId and stepElement are required');
      }

      if (this.states.indexOf(stateId) === -1) {
        this._throwError('stateId=' + stateId + ' is not defined in states');
      }

      if (afterStateId && !this.isStateAdded(afterStateId)) {
        this._throwError('cannot find afterStateId=' + afterStateId);
      }

      var newStep = this._createStep(stateId, stepElement);

      if (this.isStateAdded(stateId)) {
        this._replaceStep(newStep, stateId);
      } else if (!afterStateId && this.steps.length === 0) {
        this.steps.push(newStep);
      } else if (!afterStateId && this.steps.length > 0) {
        this.steps.replace([newStep]);
      } else {
        this._addStepAfter(newStep, afterStateId);
      }

      this.changeState(stateId);
    }

    /**
     * Removes any steps after the specified state id.
     * @param afterStateId
     */

  }, {
    key: 'removeSteps',
    value: function removeSteps(_ref2) {
      var afterStateId = _ref2.afterStateId;

      if (!afterStateId) {
        this._throwError('afterStateId is required');
      }

      // lose any steps after the current step
      var stepsCopy = this._sliceSteps(afterStateId);
      this.steps.replace(stepsCopy);

      // update the current stateId to match the last step's stateId
      var lastStep = this.steps[this.steps.length - 1];
      this.changeState(lastStep.stateId);
    }
  }, {
    key: '_createStep',
    value: function _createStep(stateId, stepElement) {
      var newStep = {
        stateId: stateId,
        stepElement: stepElement
      };

      // each time a new step instance is created we want fresh React state
      // assign a unique ID to the React element's key to force a remount
      newStep.stepElement = _react2.default.cloneElement(newStep.stepElement, { key: randomId() });
      return newStep;
    }
  }, {
    key: '_addStepAfter',
    value: function _addStepAfter(newStep, afterStateId) {
      var stepsCopy = this._sliceSteps(afterStateId);
      stepsCopy.push(newStep);
      this.steps.replace(stepsCopy);
    }
  }, {
    key: '_replaceStep',
    value: function _replaceStep(newStep, targetStateId) {
      // grab all steps up to but not including the target
      // then add the new step and commit
      var targetIndex = this._findStepIndex(targetStateId);
      var stepsCopy = this.steps.slice(0, targetIndex);
      stepsCopy.push(newStep);
      this.steps.replace(stepsCopy);
    }
  }, {
    key: 'changeState',
    value: function changeState(stateId) {
      if (!stateId) {
        this._throwError('stateId is required');
      }

      var currentStep = this.steps[this.steps.length - 1];

      if (this.stateId === stateId && currentStep.stateId === stateId) {
        console.warn('stateId already set to ' + stateId);
      }

      currentStep.stateId = stateId;
      this.stateId = stateId;
    }
  }, {
    key: '_findStep',
    value: function _findStep(stateId) {
      var matches = this.steps.filter(function (step) {
        return step.stateId === stateId;
      });

      if (matches.length === 1) {
        return matches[0];
      }

      return null;
    }
  }, {
    key: '_findStepIndex',
    value: function _findStepIndex(stateId) {
      var step = this._findStep(stateId);
      return this.steps.indexOf(step);
    }

    /**
     * Return all steps up to and including the step with specified stateId.
     * @param stateId
     * @returns {Array}
     * @private
     */

  }, {
    key: '_sliceSteps',
    value: function _sliceSteps(stateId) {
      var targetIndex = this._findStepIndex(stateId);
      return this.steps.slice(0, targetIndex + 1);
    }
  }, {
    key: 'isStateAdded',
    value: function isStateAdded(stateId) {
      return this._findStep(stateId) !== null;
    }
  }, {
    key: 'setPlaceholders',
    value: function setPlaceholders(placeholders) {
      var array = [];

      if (typeof placeholders === 'string') {
        array.push(placeholders);
      } else if (placeholders) {
        array = placeholders;
      }

      this.placeholders.replace(array);
    }
  }, {
    key: 'completeFlow',
    value: function completeFlow(payload) {
      if (this.listener && this.listener.onComplete) {
        this.listener.onComplete(payload);
      }
    }
  }, {
    key: '_setupStates',
    value: function _setupStates() {
      this.states = this.getStates();
    }
  }, {
    key: '_setupInitialStep',
    value: function _setupInitialStep() {
      var initial = this.getInitialStep();
      this.renderStep(initial);
    }
  }, {
    key: '_reset',
    value: function _reset() {
      // these collections should be observable but we don't want elements themselves to be observable
      // (some of them are React elements and making them observable leads to strange runtime errors)
      this.stateId = null;
      this.states = [];
      this.steps = (0, _mobx.observable)((0, _mobx.asFlat)([]));
      this.placeholders = (0, _mobx.observable)((0, _mobx.asFlat)([]));

      this.listener = null;
    }
  }, {
    key: '_throwError',
    value: function _throwError(errorString) {
      console.error(errorString);
      throw new Error(errorString);
    }
  }, {
    key: 'redirectTimeout',
    get: function get() {
      return 2000;
    }
  }, {
    key: 'activeIndex',
    get: function get() {
      return this.steps.length > 0 ? this.steps.length - 1 : 0;
    }
  }]);

  return FlowManager;
}(), (_applyDecoratedDescriptor(_class.prototype, 'activeIndex', [_mobx.computed], Object.getOwnPropertyDescriptor(_class.prototype, 'activeIndex'), _class.prototype), _descriptor = _applyDecoratedDescriptor(_class.prototype, 'stateId', [_mobx.observable], {
  enumerable: true,
  initializer: function initializer() {
    return null;
  }
}), _applyDecoratedDescriptor(_class.prototype, 'renderStep', [_mobx.action], Object.getOwnPropertyDescriptor(_class.prototype, 'renderStep'), _class.prototype), _applyDecoratedDescriptor(_class.prototype, 'removeSteps', [_mobx.action], Object.getOwnPropertyDescriptor(_class.prototype, 'removeSteps'), _class.prototype), _applyDecoratedDescriptor(_class.prototype, '_addStepAfter', [_mobx.action], Object.getOwnPropertyDescriptor(_class.prototype, '_addStepAfter'), _class.prototype), _applyDecoratedDescriptor(_class.prototype, '_replaceStep', [_mobx.action], Object.getOwnPropertyDescriptor(_class.prototype, '_replaceStep'), _class.prototype), _applyDecoratedDescriptor(_class.prototype, 'changeState', [_mobx.action], Object.getOwnPropertyDescriptor(_class.prototype, 'changeState'), _class.prototype), _applyDecoratedDescriptor(_class.prototype, 'setPlaceholders', [_mobx.action], Object.getOwnPropertyDescriptor(_class.prototype, 'setPlaceholders'), _class.prototype), _applyDecoratedDescriptor(_class.prototype, '_setupInitialStep', [_mobx.action], Object.getOwnPropertyDescriptor(_class.prototype, '_setupInitialStep'), _class.prototype), _applyDecoratedDescriptor(_class.prototype, '_reset', [_mobx.action], Object.getOwnPropertyDescriptor(_class.prototype, '_reset'), _class.prototype)), _class);
exports.default = FlowManager;