'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = StepIndicator;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _designLanguage = require('@jenkins-cd/design-language');

var _FlowStepStatus = require('./FlowStepStatus');

var _FlowStepStatus2 = _interopRequireDefault(_FlowStepStatus);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Visual indicator that displays a workflow step as either completed, active, incomplete or error state.
 * Can show partial completion via 'percentage' prop.
 */
function StepIndicator(props) {
  var newProps = {};
  newProps.width = newProps.height = '32';

  if (!isNaN(props.percentage) && props.percentage >= 0 && props.status !== _FlowStepStatus2.default.COMPLETE) {
    newProps.result = 'running';
    newProps.percentage = props.percentage;
  } else if (props.status === _FlowStepStatus2.default.COMPLETE) {
    newProps.result = 'success';
  } else if (props.status === _FlowStepStatus2.default.ERROR) {
    newProps.result = 'failure';
  } else {
    newProps.result = 'not_built';
  }

  return _react2.default.createElement(
    'div',
    { className: 'step-indicator-component' },
    _react2.default.createElement(_designLanguage.StatusIndicator, newProps)
  );
}

StepIndicator.propTypes = {
  status: _propTypes2.default.oneOf(_FlowStepStatus2.default.values()),
  percentage: _propTypes2.default.number
};

StepIndicator.defaultProps = {
  status: 'incomplete',
  percentage: -1
};